/**
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * beck et al. projects GmbH
 * Theresienhoehe 13
 * 80336 Munich
 * Phone: +49-89-5442530
 * mailto: info@bea.de
 */
package de.bsvrz.ste.stealgsba.algorithmen.lib;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.exceptions.LoggerException;
import de.bsvrz.sys.funclib.losb.exceptions.SenderException;
import de.bsvrz.sys.funclib.losb.kernsoftware.ConnectionManager;
import de.bsvrz.sys.funclib.losb.messages.ErrorMessage;
import de.bsvrz.sys.funclib.losb.util.Util;

/**
 * Klasse die das Senden ohne Bercksichtigung der 
 * Sendesteuerung durchfhrt. Nach dem Senden der Daten meldet sich der Sender
 * sofort beim Datenverteiler ab.
 * <br>
 * <b>WICHTIG</b><br>
 * Wenn versucht wird, etwas ohne angemeldeten Empfnger Quelle zu senden, dann wird (korrekterweise)
 * ein Fehler gemeldet. 
 * Problematisch wird es, wenn sofort danach eine Quellen-/Empfngeranmeldung durchgefhrt und das Senden
 * wiederholt wird. <b>Dies fhrt zu einem Fehler<b>, da die Sendesteuerung das Senden immer noch 
 * nicht gestattet. Daher sollte eine Pause zwischen zwei Sendeversuchen liegen. (Siehe JUnit-Tests).
 * 
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision: 1.1 $ / $Date: 2012/04/27 09:19:02 $ / ($Author: Asal $)
 */

public class SenderOhneStatusRueckmeldung implements ClientSenderInterface
{
	private Data data;
	private SystemObject receiver;
	private ClientDavInterface dav;
	private DataDescription dataDescription;
	private static final Debug debug = Debug.getLogger();
	
	/**
	 * Erzeugt einen Sender und sendet die Daten als einfacher Sender.
	 * Die Methode wartet solange, bis die Daten gesendet wurden, oder es zu einem Fehler kam.
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data Daten, die versendet werden sollen
	 * @param timeout	Maximale Wartezeit in ms. Bei <code>0</code> unbegrenzt.
	 * @throws SenderException Fehler beim Senden.
	 * @throws RuntimeException Kann bei Fehler geworfen werden.
	 */
	public static void sendWait(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data, long timeout) throws SenderException
	{
		send(dav, receiver, dataDescription, data, false, timeout);
	}

	/**
	 * Erzeugt einen Sender und sendet die Daten als Quelle.
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data Daten, die versendet werden sollen
	 * @return Quelle
	 * @throws SenderException Fehler beim Senden.
	 */
	public static SenderOhneStatusRueckmeldung source(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data) throws SenderException
	{
		SenderOhneStatusRueckmeldung sender = new SenderOhneStatusRueckmeldung(dav, receiver, dataDescription, data, true);
		return sender;
	}
	
	/**
	 * Erzeugt einen Sender und sendet die Daten als einfache Quelle.
	 * Die Methode wartet solange, bis die Daten gesendet wurden, oder es zu einem Fehler kam.
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data Daten, die versendet werden sollen
	 * @param timeout	Maximale Wartezeit in ms. Bei <code>0</code> unbegrenzt.
	 * @throws SenderException Falls es beim Senden zu einem Fehler kommt.
	 * @throws RuntimeException Kann bei Fehler geworfen werden.
	 */
	public static void sourceWait(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data, long timeout) throws SenderException
	{
		send(dav, receiver, dataDescription, data, true, timeout);
	}
	
	/**
	 * Initialisiert das Lock-Objekt.
	 */
	private SenderOhneStatusRueckmeldung()
	{
	}
	
	/**
	 * Erzeugt den Sender und sendet die Daten.
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver	Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data		Daten, die versendet werden sollen
	 * @param source	True wenn die Daten als Quelle versendet werden sollen. Ist source == false meldet sich {@link SenderOhneStatusRueckmeldung} als einfacher Sender an.
	 * @throws SenderException Falls es bei der Sendeanmeldung zu einem Fehler kommt.
	 */
	public SenderOhneStatusRueckmeldung(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data, boolean source) throws SenderException
	{
		this();
		init(dav, receiver, dataDescription, data, source);
	}
	
	/**
	 * Objektinitialisierung.
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver	Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data		Daten, die versendet werden sollen
	 * @param source	True wenn die Daten als Quelle versendet werden sollen. Ist source == false meldet sich {@link SenderOhneStatusRueckmeldung} als einfacher Sender an.
	 * @throws SenderException Falls es bei der Sendeanmeldung zu einem Fehler kommt.
	 */
	private void init(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data, boolean source) throws SenderException
	{
		this.dav = dav;
		this.data = data;
		this.receiver = receiver;
		this.dataDescription = dataDescription;
		
		SenderRole role = (source) ? SenderRole.source() : SenderRole.sender();
		try
		{
			ConnectionManager.subscribeSender(dav, this, receiver, dataDescription, role);
		}
		catch (OneSubscriptionPerSendData e)	//Es existiert bereits eine Anmeldung. Strt nicht weiter.
		{
			String msg = receiver.getNameOrPidOrId() + ":"+ dataDescription;
			debug.warning(ErrorMessage.MULIPLE_SUBSCRIPTIONS+msg);
		}
		catch (RuntimeException re)
		{
			throw new SenderException(ErrorMessage.CAN_NOT_SUBSCRIBE + Util.getStackTrace(re), LoggerException.WARNING);
		}
	}
	
	/**
	 * Erzeugt einen Sender und sendet die Daten.
	 * Die Methode wartet solange, bis die Daten gesendet wurden, oder es zu einem Fehler kam.
	 * Kann <code>RuntimException</code> werfen!
	 * @param dav		Verbindung zum Datenverteiler
	 * @param receiver Empfnger der Daten
	 * @param dataDescription DataDescription der zu versendenden Daten
	 * @param data Daten, die versendet werden sollen
	 * @param source	True wenn als Quelle gesendet werden soll. False, wenn als einfacher Sender gesendet werden soll.
	 * @param timeout	Maximale Wartezeit in ms. Bei <code>0</code> unbegrenzt.
	 * @throws SenderException Falls es whrend des Sendens zu einem Fehler kam.
	 * @throws RuntimeException Kann bei Fehler geworfen werden.
	 */
	private static void send(ClientDavInterface dav, SystemObject receiver, DataDescription dataDescription, Data data, boolean source, long timeout) throws SenderException
	{
		SenderOhneStatusRueckmeldung sender = new SenderOhneStatusRueckmeldung();
		try
		{
			sender.init(dav, receiver, dataDescription, data, source);
		}
		catch (SenderException e)
		{
			debug.error(e.getMessage());
		}
	}

	/**
	 * Sendesteuerung. Callback Methode fr den Datenverteiler.
	 * Wird der Versand von Daten erlaubt, so werden die Daten sofort abgesetzt. Danach
	 * meldet sich der Sender unverzglich ab.
	 * Wird der Versand der Daten nicht erlaubt, so werden sie verworfen.
	 * @param object Das in der zugehrigen Sendeanmeldung angegebene Objekt, auf das sich die Sendesteuerung bezieht.
	 * @param dataDescription Beschreibende Informationen zu den angemeldeten Daten auf die sich die Sendesteuerung bezieht.
	 * @param state Status der Sendesteuerung. Kann einen der Werte START_SENDING, STOP_SENDING, STOP_SENDING_NO_RIGHTS, STOP_SENDING_NOT_A_VALID_SUBSCRIPTION enthalten.
	 */
	public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
	{
		try
		{
			switch (state)
			{		
				case ClientSenderInterface.START_SENDING:
					ResultData rd = new ResultData(receiver, this.dataDescription, System.currentTimeMillis(), data);
					try
					{
						dav.sendData(rd);
					}
					catch (SendSubscriptionNotConfirmed e)
					{
						//sollte nie aufgerufen werden, da ja gerade berprft wurde, ob Sender senden darf
						String msg = ErrorMessage.SENDING_NOT_ALLOWED+e.getMessage();
						debug.error(msg);
					}
					catch (RuntimeException e)
					{
						String msg = ErrorMessage.CAN_NOT_SEND + Util.getStackTrace(e);
						debug.error(msg);
					}
					break;
				case ClientSenderInterface.STOP_SENDING:
					break;
				default:
				{
					String msg  = ErrorMessage.SENDING_NOT_ALLOWED +object.getNameOrPidOrId()+":"+dataDescription + ", Sendsteuerung: " + state;
					debug.error(msg);
				}
			}
		}
		finally
		{
//			ConnectionManager.unsubscribeSender(dav, this, object, dataDescription);
		}
	}

	/**
	 * Sendesteuerung ist erwnscht. Wird vom Datenverteiler aufgerufen.
	 * Liefert deshalb immer true zurck.
	 * @param object Das in der zugehrigen Sendeanmeldung angegebene System-Objekt.
	 * @param dataDescription Die in der zugehrigen Sendeanmeldung angegebenen beschreibenden Informationen der angemeldeten Daten.
	 * @return <code>true</code>, falls Sendesteuerungen gewnscht sind, sonst <code>false</code>.
	 * @see ClientSenderInterface#isRequestSupported(SystemObject, DataDescription)
	 */
	public boolean isRequestSupported(SystemObject object, DataDescription dataDescription)
	{
		return true;
	}
}
