/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Diese Klasse speichert die Analysedaten einer Ursacheneinheit, bei der die Situationsbestimmung
 * auf Messquerschnitten basiert.
 *
 * @version $Revision: 1.4 $ / $Date: 2015/08/26 13:57:08 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class SituationAnalyseMq implements Cloneable
{
	/**
	 * Status Einschaltkriterium.
	 */
	private boolean m_einschaltkriterium;
	/**
	 * Status Ausschaltkriterium.
	 */
	private boolean m_ausschaltkriterium;
	/**
	 * Ergebnis der Situationsanalyse pro Messquerschnitt.
	 */
	private Vector<SituationAnalyseMqWerte> m_situationAnalyse;
	
	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public SituationAnalyseMq()
	{		
	}
	
	/**
	 * Konstruktor mit bernahme der Daten.
	 * 
	 * @param rdata zu bernehmender Datensatz.
	 */
	public SituationAnalyseMq(ResultData rdata)
	{
		if (rdata != null)
		{
			if (rdata.hasData())
			{
				Data data = rdata.getData();
				
				m_einschaltkriterium = data.getScaledValue("Einschaltkriterium").getValueText().equals("Ja");
				m_ausschaltkriterium = data.getScaledValue("Ausschaltkriterium").getValueText().equals("Ja");

				for (int i = 0; i < data.getArray("Status").getLength(); i++)
				{					
					Data element = data.getArray("Status").getItem(i);
					if (element != null)
					{
						addAnalyseMq(element.getReferenceValue("MessQuerschnitt").getSystemObject(), element.getScaledValue("Aktiv").getValueText().equals("Ja"));
					}			
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() 
	{
		StringBuffer string = new StringBuffer();
		
		string.append("EinKrit=" + m_einschaltkriterium +
									" AusKrit=" + m_ausschaltkriterium + 
									" Status MQs " + m_situationAnalyse);
		
		return string.toString();
  }

	/**
	 * Fgt zu der Liste der einen neuen Datensatz mit den bergebenen Werten hinzu.
	 * 
	 * @param mq Systemobjekt Messquerschnitt.
	 * @param status Status der Situation;
	 */
	public void addAnalyseMq(SystemObject mq, boolean status)
	{
		if (m_situationAnalyse == null)
		{
			m_situationAnalyse = new Vector<SituationAnalyseMqWerte>();
		}
		
		SituationAnalyseMqWerte werte = new SituationAnalyseMqWerte();
		werte.setMessQuerschnitt(mq);
		werte.setAktiv("Nein");
		if (status)
			werte.setAktiv("Ja");
		
		m_situationAnalyse.add(werte);
	}
	
	/**
	 * Lscht die Liste der Situationsanalyse.
	 */
	public void clear()
	{
		if (m_situationAnalyse != null)
		 m_situationAnalyse.clear();
	}
	
	/**
	 * Gibt die Lnge des Vektors zurck.
	 * 
	 * @return Lnge des Vektors.
	 */
	public int getLength()
	{
		if (m_situationAnalyse != null)
		{
			return m_situationAnalyse.size();
		}
		return 0;
	}
	/**
	 * Gibt den Eintrag des bergebenen Index zurck.
	 * 
	 * @return Werte entsprechend dem Index.
	 */
	public SituationAnalyseMqWerte getItem(int index)
	{
		if (m_situationAnalyse != null)
		{
			return m_situationAnalyse.get(index);
		}
		return null;
	}
	/**
	 * Innere Klasse zum Speichern der Werte pro Messquerschnitt.
	 *
	 * @version $Revision: 1.4 $ / $Date: 2015/08/26 13:57:08 $ / ($Author: Asal $)
	 *
	 * @author Swarco Traffic Systems GmbH
	 * @author Rolf Asal
	 *
	 */
	public class SituationAnalyseMqWerte implements Cloneable
	{
		/**
		 * Objekt Messquerschnitt.
		 */
		private SystemObject m_messQuerschnitt;
		/**
		 * Status der Situation.
		 */
		private String m_aktiv;
		
		/**
		 * Default-Konstruktor ohne weitere Funktion.
		 */
		public SituationAnalyseMqWerte()
		{			
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() 
		{
			StringBuffer string = new StringBuffer();
			
			string.append(" MQ " + m_messQuerschnitt +
										" Aktiv " + m_aktiv); 
			
			return string.toString();
	  }

		/**
		 * Gibt das Objekt Messquerschnitt zurck.
		 * 
		 * @return Objekt Messquerschnitt.
		 */
		public SystemObject getMessQuerschnitt()
		{
			return m_messQuerschnitt;
		}

		/**
		 * Setzt das Objekt Messquerschnitt.
		 * 
		 * @param messQuerschnitt zu setzendes Objekt Messquerschnitt.
		 */
		public void setMessQuerschnitt(SystemObject messQuerschnitt)
		{
			m_messQuerschnitt = messQuerschnitt;
		}

		/**
		 * gibt den Status der Situation zurck.
		 * 
		 * @return Status der Situation.
		 */
		public String getAktiv()
		{
			return m_aktiv;
		}

		/**
		 * Setzt den Status der Situation.
		 * 
		 * @param aktiv zu setzender Status der Situation.
		 */
		public void setAktiv(String aktiv)
		{
			m_aktiv = aktiv;
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#equals(java.lang.Object)
		 */
		@Override
		public boolean equals(Object obj)
		{
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			SituationAnalyseMqWerte other = (SituationAnalyseMqWerte) obj;
			if (m_aktiv == null)
			{
				if (other.m_aktiv != null)
					return false;
			} else if (!m_aktiv.equals(other.m_aktiv))
				return false;
			if (m_messQuerschnitt == null)
			{
				if (other.m_messQuerschnitt != null)
					return false;
			} else if (!m_messQuerschnitt.equals(other.m_messQuerschnitt))
				return false;
			return true;
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#clone()
		 */
		@Override
		protected Object clone()
		{
			try
			{
				return (SituationAnalyseMqWerte)super.clone();
			}
			catch (CloneNotSupportedException e)
			{
				e.printStackTrace();
			}

			return null;
		}

	}
	
	/**
	 * @return the situationAnalyse
	 */
	public Vector<SituationAnalyseMqWerte> getSituationAnalyse()
	{
		return m_situationAnalyse;
	}

	/**
	 * @param situationAnalyse the situationAnalyse to set
	 */
	public void setSituationAnalyse(Vector<SituationAnalyseMqWerte> situationAnalyse)
	{
		m_situationAnalyse = situationAnalyse;
	}

	/**
	 * Prft den Status des Einschaltkriteriums.
	 * 
	 * @return true wenn Einschaltkriteriums wahr, ansonsten false.
	 */
	public boolean isEinschaltkriterium()
	{
		return m_einschaltkriterium;
	}

	/**
	 * Setzt den Status des Einschaltkriteriums.
	 * @param einschaltkriterium zu setzender Status.
	 */
	public void setEinschaltkriterium(boolean einschaltkriterium)
	{
		m_einschaltkriterium = einschaltkriterium;
	}

	/**
	 * Prft den Status des Ausschaltkriteriums.
	 * 
	 * @return true wenn Ausschaltkriteriums wahr, ansonsten false.
	 */
	public boolean isAusschaltkriterium()
	{
		return m_ausschaltkriterium;
	}

	/**
	 * Setzt den Status des Ausschaltkriteriums.
	 * @param ausschaltkriterium zu setzender Status.
	 */
	public void setAusschaltkriterium(boolean ausschaltkriterium)
	{
		m_ausschaltkriterium = ausschaltkriterium;
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone()
	{
		try
		{
			return (SituationAnalyseMq)super.clone();
		}
		catch (CloneNotSupportedException e)
		{
			e.printStackTrace();
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		SituationAnalyseMq other = (SituationAnalyseMq) obj;
		if (m_ausschaltkriterium != other.m_ausschaltkriterium)
			return false;
		if (m_einschaltkriterium != other.m_einschaltkriterium)
			return false;
		if (m_situationAnalyse == null)
		{
			if (other.m_situationAnalyse != null)
				return false;
		} else if (!m_situationAnalyse.equals(other.m_situationAnalyse))
			return false;
		return true;
	}
	
//	/* (non-Javadoc)
//	 * @see java.lang.Object#equals(java.lang.Object)
//	 */
//	@Override
//	public boolean equals(Object obj)
//	{
//		if (this == obj)
//			return true;
//		if (obj == null)
//			return false;
//		if (getClass() != obj.getClass())
//			return false;
//		SituationAnalyseMq other = (SituationAnalyseMq) obj;
//		if (m_ausschaltkriterium != other.m_ausschaltkriterium)
//			return false;
//		if (m_einschaltkriterium != other.m_einschaltkriterium)
//			return false;
//		if (m_situationAnalyse == null)
//		{
//			if (other.m_situationAnalyse != null)
//				return false;			
//		} 
//		else
//		{
//			if (m_situationAnalyse.size() != other.m_situationAnalyse.size())				
//				return false;
//			
//			for (int i = 0; i < m_situationAnalyse.size(); i++)
//			{
//				if (!m_situationAnalyse.get(i).equals(other.m_situationAnalyse.get(i)))
//					return false;
//			}
//		}
//		
//		return true;
//	}
}
